% ------------------------------------------------------------------------
% [1]: Nora Horanyi, Zoltan Kato, Generalized Pose Estimation from Line 
% Correspondences with Known Vertical Direction, In Proceedings of 
% International Conference on 3D Vision, Qingdao, China, 2017, IEEE
% All rights reserved.
% ------------------------------------------------------------------------

% published:  2017-12-05, using Matlab 9.2.0.556344 (R2017a)
% updated:  2020-04-27, using Matlab (R2018b)
% the code has been updated by hichem Abdellali

% Important!
% ------------------------------------------------------------------------
% Please cite our [1] publication whenever you use the implementation.


% Introduction
% ------------------------------------------------------------------------
% This software package contains the algorithm as described in paper [1].


% Installation
% ------------------------------------------------------------------------

% This package does not require any installation. Package contains all
% necessary functions. the solver needs endpoints from 2D-3D lines
% coorespandances (an example is provided),

% ------------------------------------------------------------------------
% -------------------------------- NOTE ----------------------------------
% ------------------------------------------------------------------------
% the algorithm expects normalized coordinates on the unit 
% sphere. If our camera is a calibrated perspective camera and we have a 
% 2D homogenous coordinate 'x' then using K:
% temp = inv(K) * x;
% x_sphere = temp ./ norm(temp);


% ------------------------------------------------------------------------
% ---------------------------- Main input --------------------------------
% ------------------------------------------------------------------------
% input.Camera(i).Line.start = normalized coordinates on the unit sphere of 
% the image line segment startpoints.
% input.Camera(i).Line.end = normalized coordinates on the unit sphere of 
% the image line segment endpoints.
% input.line3D.start = start points of the 3D line segments in world coordinates.
% input.line3D.end = end points of the 3D line segments in world coordinates.
% camera type : 
%               --> 'PerspAbs' or 'Persp'  for perspective camera
%               --> 'OmniAbs' or 'Omni' for Omnidirectional camera

% ------------------------------------------------------------------------
% -------------------------additional input ------------------------------
% ------------------------------------------------------------------------
% camNum: number of camera. 
% lineNum : number of 2D-3D line pairs to be used.


% ------------------------------------------------------------------------
% ----------------------------- output -----------------------------------
% ------------------------------------------------------------------------
% poses : ('Absolute pose') 4*4 matrix acts from the world coordinate 
% system to the reference camera. The optimal pose is ranked as the first
% solution in the returned list based on the backprojection error.


% Note: we select the best solution using the line backprojection
% function working on the unit sphere and a geometric constaint that the 
% camera should face the scene. for this we assumed that the camera is 
% looking for the z+





clear all
restoredefaultpath
dbstop if error

addpath Functions
addpath Solver

% ------------------------------------------------------------------------
% ------------------------------ Tests -----------------------------------
% ------------------------------------------------------------------------
% run this script "HOW_TO_RUN.m". two examples will be loaded from: 


% input_persp.mat
% it contains a configuration of 3 cameras 
% camera 1 : perspective reference camera
% camera 2 : perspective camera
% camera 3 : Omnidirectional camera
% and 60 2D-3D line correspandances, 
load('input_persp.mat');
camNum = 3;
lineNum = 60;
[Pose1]  = gPnLup(input,camNum,lineNum);



% input_omni.mat
% it contains a configuration of 3 cameras 
% camera 1 : Omnidirectional reference camera
% camera 2 : Omnidirectional camera
% camera 3 : perspective camera
% and 60 2D-3D line correspandances, 
load('input_omni.mat');
camNum = 3;
lineNum = 60;
[Pose2]  = gPnLup(input,camNum,lineNum);










